using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Collections.Specialized;

namespace Interval.Examples.ServerControls
{
	/// <summary>
	/// Serverov ovldac prvek pro demonstraci udlost
	/// </summary>
	[DefaultEvent("Click"),
	DefaultProperty("Text"),
	ToolboxData("<{0}:SampleEventsControl runat=server></{0}:SampleEventsControl>")]
	public class SampleEventsControl : System.Web.UI.WebControls.WebControl, IPostBackDataHandler,IPostBackEventHandler
	{

		#region Events
		public event EventHandler Click;
		public event EventHandler Change;
		#endregion Events
		#region Constructors
		#endregion Constructors
		
		/// <summary>
		/// Text v textboxu
		/// </summary>
		#region properties
		
		[Bindable(true),
		Category("Appearance"),
		DefaultValue("")]
		public string TextBoxText
		{
			get
			{
				//Zskn hodnoty z ViewState
				string vtext = (string) ViewState["textBoxText"];
				
			    //Pokud hodnota nebyla uloena do ViewState, vra przdn etzec.
				if (vtext == null)
					return String.Empty;
					//Jinak vra hodnotu z ViewState
				else
					return vtext;
			}
			set
			{
				ViewState["textBoxText"] = value;
			}
		}
		[Bindable(true),
		Category("Appearance"),
		DefaultValue("")]
		public string Text
		{
			get
			{
				//Zskn hodnoty z ViewState
				string vtext = (string) ViewState["Text"];
				
				//Pokud hodnota nebyla uloena do ViewState, vra przdn etzec.
				if (vtext == null)
					return String.Empty;
					//Jinak vra hodnotu z ViewState
				else
					return vtext;
			}

			set
			{
				ViewState["Text"] = value;
			}
		}	
		
		#endregion properties
		
		#region public methods
		/// <summary>
		/// Explicitn implementace metody LoadPostData
		/// </summary>
		/// <param name="postDataKey">Uniktn kl</param>
		/// <param name="postCollection">POST hodnoty z WWW formulre</param>
		/// <returns>True - m dojt k vyvoln udlosti Change, False - nem dojt k vyvoln udlosti Change</returns>
		bool IPostBackDataHandler.LoadPostData(string postDataKey, NameValueCollection postCollection)
		{
			
			//Hodnota z WWW formule
			string newValue = postCollection[postDataKey];
			//Pedchoz hodnota
			string oldValue = TextBoxText;
			//Pokud se text zmnil, chceme vyvolat udlost Change
			if (newValue != oldValue)
			{
				TextBoxText=newValue;
				return true;
			}
			else
				return false;

		}
		/// <summary>
		/// Explicitn implementace metody RaisePostDataChangedEvent pro vyvolvn zmnovch udlost
		/// </summary>
		void IPostBackDataHandler.RaisePostDataChangedEvent()
		{
			OnChange(new EventArgs());
		}
		
		/// <summary>
		/// Explicitn implementace metody RaisePostBackEvent pro vyvolvn aknch udlost
		/// </summary>
		/// <param name="eventArgument">Identifiktor udlosti, zde vdy przdn etzec </param>
		void IPostBackEventHandler.RaisePostBackEvent(string eventArgument)
		{
			OnClick(new EventArgs());
		}
		#endregion public methods
		
		#region protected methods
		
		
		/// <summary>
		/// V metod RenderContents vypeme obsah prvku. 
		/// </summary>
		protected override void RenderContents (HtmlTextWriter writer)
		{
			writer.AddAttribute(HtmlTextWriterAttribute.Type,"text");
			//Html atribut name m hodnotu vlastnosti UniqueId, aby ASP.NET runtime volal nmetodu LoadPostData
			writer.AddAttribute(HtmlTextWriterAttribute.Name, this.UniqueID);
			writer.AddAttribute(HtmlTextWriterAttribute.Value, this.TextBoxText);
			
			writer.RenderBeginTag(HtmlTextWriterTag.Input);
			writer.RenderEndTag();
			
			writer.WriteFullBeginTag("br/");
			writer.WriteFullBeginTag("br/");
			
			//Mapovn klientsk udlosti Onclick na sreverovou Click
			writer.AddAttribute(HtmlTextWriterAttribute.Onclick, Page.GetPostBackEventReference(this));
			writer.AddAttribute(HtmlTextWriterAttribute.Name, this.UniqueID + "_button");
			writer.RenderBeginTag(HtmlTextWriterTag.Button);
			writer.Write(this.Text);
			writer.RenderEndTag();

		}

		/// <summary>
		/// Metoda pro vyvoln udlosti <see cref="Click"/>
		/// </summary>
		/// <param name="e">Parametry udlosti</param>
		protected virtual void OnClick(EventArgs e)
		{
			if (Click != null)
				Click(this, e);
			
		}

		/// <summary>
		/// Metoda pro vyvoln udlosti <see cref="Change"/>
		/// </summary>
		/// <param name="e">Parametry udlosti</param>
		protected virtual void OnChange(EventArgs e)
		{
			if (Change != null)
				Change(this, e);
			
		}
		#endregion protected methods

		
	}
}
